#include "defs.h"

/* ceiling function */
int roundup(float num)
{
  int approx = (int)num;
  if (num == approx)
    return(approx);
  else
    return(approx+1);
}

/* floor function */
int rounddown(float num)
{
  int approx = (int)num;
  if (num < approx)
    return(approx-1);
  else
    return(approx);
}

/* given a file name and starting coordinates, dump a section of the */
/* 2D pixel array called colors, to the file in raw ppm format */
void dump_block(char *filename, int height, int width, int StartVert, 
		int StartHoriz, int maxval)
{
  int i,j, lineptr;
  FILE *to;
  char *line;
  int EndVert = StartVert + height;
  int EndHoriz = StartHoriz + width;
  int linelength = (EndHoriz - StartHoriz)*3;
  line = malloc(linelength*sizeof(char));
  to = fopen(filename, "w");
  fprintf(to, "P6\n");
  fprintf(to, "%d %d\n", width, height);
  fprintf(to, "%d\n", maxval);
  for (i=StartVert; i<EndVert; i++)
    {
      lineptr = 0;
      for (j=StartHoriz; j<EndHoriz; j++)
	{
	  line[lineptr] = colors[i][j].red;
	  line[lineptr+1] = colors[i][j].green;
	  line[lineptr+2] = colors[i][j].blue;
	  lineptr += 3;
	}
      fwrite(line, sizeof(char), linelength, to);
    }
  free(line);
  fclose(to);
}

/* iteratively call dump_block to dump the 2D pixel array colors, into */
/* a series of small raw ppm files */ 
void dump_into_blocks(char *filebase, int height, int width, int
		      maxval, int start)
{
  int i,j;
  int BlockHeight = 300;
  int BlockWidth = 300;
  int NumDown = roundup(height/BlockHeight);
  int NumAcross = roundup(width/BlockWidth);
  char filename[100];
  for (i=0; i<=NumDown; i++)
    for (j=0; j<=NumAcross; j++)
      {
	sprintf(filename, "%s.%d.%d.ppm", filebase, start+i+1, j+1);
	dump_block(filename, 
		   i==NumDown? height % BlockHeight : BlockHeight, 
		   j==NumAcross? width % BlockWidth : BlockWidth, 
		   i*BlockHeight, j*BlockWidth, maxval);
      }
}

/* read in a raw ppm file with no comments, a piece at a time.  In */
/* this case, the piece is always 300 pixels high and as wide as the */
/* image is.  Depending on the zoom level, read in every pixel, every */
/* 2nd pixel on every second line, etc.  Dump the stuff read in into */
/* raw ppm files.  */
void process_in_blocks(char *ppmname, int *height, int *width, int *maxval,
                       pixel ***grid, int power)
{
  int i,j,lineptr,heightptr,thisheight,thiswidth,numIt;
  FILE *from;
  char MagicNumber[3];
  char *line;
  int linelength;
  char filename[100];
  char *filebase = "/projects/kyk02/terrain/cpatton/test"; 
/*  char *filebase = "./test"; */

    from = fopen(ppmname, "r");
  if ((strcmp(ppmname, "/projects/kyk02/terrain/data/NYS/datafiles/landsat.albany") == 0) ||  (strcmp(ppmname, "/export/projects/kyk02/terrain/data/NYS/datafiles/landsat.albany") == 0))
    {
      /* first 4 integers are not of interest, so read them into width and
         they get overwritten */
      fread(width, sizeof(int), 1, from);
      fread(width, sizeof(int), 1, from);
      fread(width, sizeof(int), 1, from);
      fread(width, sizeof(int), 1, from); 
      fread(width, sizeof(int), 1, from);
      fread(height, sizeof(int), 1, from);
      MagicNumber[0] = 'P';
      MagicNumber[1] = '6';
      *maxval = 255;
    }
  else
    {
      fgets( MagicNumber, 3, from);
      fscanf(from, "%d", width);
      fscanf(from, "%d", height);
      fscanf(from, "%d\n", maxval);
    }

  linelength = *width*3;
  line = malloc(linelength * sizeof(char));
  heightptr = 0;
  numIt = -1;

/* do the processing, magnifying, and dumping in chunks, to a 300-line */
/* section of the original ppm file at a time since the whole thing */
/* might not fit in memory at once */
  while(heightptr < *height)
    {
      numIt++;
      thisheight = ((*height - heightptr) >= 300*power)? 300
	: rounddown((*height - heightptr)/power);
      thiswidth = rounddown(*width/power);
      *grid = malloc(thisheight * sizeof(pixel*));
      for (i=0; i<thisheight; i++)
	(*grid)[i] = malloc(*width * sizeof(pixel));

      /* read in the section */      
      for (i=0; i<thisheight; i++)
	{
	  for (j=0; j<power; j++)
	    fread(line, sizeof(char), linelength, from);
	  lineptr = 0;
	  for (j=0; j<*width; j++)
	    {
	      (*grid)[i][j].red = line[lineptr];
	      (*grid)[i][j].green = line[lineptr+1];
	      (*grid)[i][j].blue = line[lineptr+2]; 
	      lineptr += 3*power;
	    }
	}

      /* do the preprocessing stuff on the section */
      if (thiswidth > 300)
	{
	  sprintf(filename, "%s_%d", filebase, (int)log2((double)power));
	  dump_into_blocks(filename, thisheight, thiswidth, *maxval, numIt);
	}
      else
	{
	  sprintf(filename, "%s_%d.%d.1.ppm", filebase, 
		  (int)log2((double)power), numIt+1);
	  dump(filename, thisheight, thiswidth, *maxval, *grid);
	}
      
      cleanup(*grid, thisheight);

      /* test for termination of while loop */
      if (thisheight == 300)
	heightptr += thisheight*power;
      else
	break;
    }
  free(line);
  fclose(from);
}


	

